//------------------------------------------------------------------------------
//
// COPYRIGHT:
// ----------
// Copyright (C) International Business Machines Corp., 1996.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM
// product and is provided to you solely for the purpose of
// assisting you in the development of your applications.  The
// code is provided "AS IS", without warranty of any kind.  IBM
// shall not be liable for any damages arising out of your use of
// the sample code, even if they have been advised of the
// possibility of such damages.
//
//------------------------------------------------------------------------------
//
//  defgram.cpp : Implements methods of the DefineGrammar class.
//                This is a Windows dialog class interacts with a user to
//                define a dynamically defined vocaulary (or grammar).
//
//                The dialog uses a VTProxy object which is used for all 
//                communications with the speech engine. It is the VTProxy 
//                class which implements the calls to SMAPI functions.  
//
//------------------------------------------------------------------------------

#include "stdafx.h"
#include "eReco.h"
#include "defGram.h"
#include "smapi.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//------------------------------------------------------------------------------
//  DefineGrammar()
//------------------------------------------------------------------------------
//
//  MFC Generated Constructor for the DefineGrammar Windows dialog.
//
//------------------------------------------------------------------------------
DefineGrammar::DefineGrammar(VTProxy   &aVTProxy,
                             CWnd* pParent /*=NULL*/)
	: m_pxySMAPI(aVTProxy),
     CDialog(DefineGrammar::IDD, pParent)
{
	//{{AFX_DATA_INIT(DefineGrammar)
	//}}AFX_DATA_INIT
}


//------------------------------------------------------------------------------
//  DoDataExchange()
//------------------------------------------------------------------------------
//
//  MFC Generated Windows data exchange method.
//
//------------------------------------------------------------------------------
void DefineGrammar::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DefineGrammar)
	DDX_Control(pDX, IDC_VOCABULARY_WORDS, m_szVocabularyWords);
	DDX_Control(pDX, IDC_VOCABULARY_NAME, m_szVocabularyName);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(DefineGrammar, CDialog)
	//{{AFX_MSG_MAP(DefineGrammar)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//------------------------------------------------------------------------------
//  OnOK()
//------------------------------------------------------------------------------
//
//  Message handler for when the "OK" button is pressed
//
//------------------------------------------------------------------------------
void DefineGrammar::OnOK() 
{
   SM_VOCWORD  **pVocPtrs;      // Dynamic array of pointers to 
                                // vocabulary structures
   int         i, iNumWords; 
   int         tempSize;
   char        *tempStr;
   int         MAXSIZE = 100;
   char        tempVocabName[100];
   BOOL        success;

   // define the dynamic vocabulary to the speech engine
	iNumWords = m_szVocabularyWords.GetLineCount(); 
   pVocPtrs = (SM_VOCWORD **)malloc(sizeof(SM_VOCWORD *) * iNumWords);

	for (i=0; i < iNumWords; i++)
   {
      // allocate & build vocabulary word structures
      pVocPtrs[i] = (SM_VOCWORD *)malloc(sizeof(SM_VOCWORD));
      pVocPtrs[i]->flags = 0;            // reserved value
      
      tempStr = (char *)malloc( MAXSIZE );
      tempSize = m_szVocabularyWords.GetLine(i, tempStr, 100);
      tempStr[tempSize] = '\0';
      
      pVocPtrs[i]->spelling_size = tempSize;
      pVocPtrs[i]->spelling = tempStr;

   }

   // Define the vocabulary with SMAPI
   tempSize = m_szVocabularyName.GetWindowText( tempVocabName, 99 );
   success = m_pxySMAPI.defineVocabulary( tempVocabName,
                                          iNumWords,
                                          pVocPtrs);
   // free up storage no longer needed
   for (i=0; i < iNumWords; i++)
   {
      free( pVocPtrs[i]->spelling );
      free( pVocPtrs[i]);

   }
   free( pVocPtrs );

   // store the vocabulary name and exit the dialog
   m_vocabularyName = tempVocabName;

   // finish up
   if ( !success )
   {
     // user needs to fix something
     CString message;
     message = "Error defining vocabulary <";
     message += tempVocabName;
     message += ">\n\n";
     if ( m_pxySMAPI.isBadWordWaiting() )
     {
        message += "Use Dictionary Builder to add pronunciations for:\n";
        message += m_pxySMAPI.getBadWords( TRUE );
        message += "\n\nthese words will not be recognized.";
     }
                      
     MessageBox( message, "Vocabulary Definition Error", MB_OK );
   }

   CDialog::OnOK();
}

//------------------------------------------------------------------------------
//  vocabularyName()
//------------------------------------------------------------------------------
//
//  return the vocabularyName attribute of this class
//
//------------------------------------------------------------------------------
const char *DefineGrammar::vocabularyName() 
{
   return ( m_vocabularyName );
}
