//------------------------------------------------------------------------------
//
// COPYRIGHT:
// ----------
// Copyright (C) International Business Machines Corp., 1996.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM
// product and is provided to you solely for the purpose of
// assisting you in the development of your applications.  The
// code is provided "AS IS", without warranty of any kind.  IBM
// shall not be liable for any damages arising out of your use of
// the sample code, even if they have been advised of the
// possibility of such damages.
//
//------------------------------------------------------------------------------
//
// gramDesc.cpp - Implements the methods for the GrammarDescriptor class.
//
//------------------------------------------------------------------------------

#include "gramDesc.h"               // Grammar Descriptor class definition

//------------------------------------------------------------------------------
//  GrammarDescriptor()
//------------------------------------------------------------------------------
//
//  Constructor for the GrammarDescriptor class.
//
//------------------------------------------------------------------------------
GrammarDescriptor::GrammarDescriptor( char const *aGrammarName )
{
  char szext[5];
 
  strcpy(m_szFSGFileName, aGrammarName);
  _splitpath(aGrammarName, NULL, NULL, m_szVocabularyName, szext);

  strlwr(szext);
  if ( !strncmp( ".fsg", szext, 4 ) )
     setType( fsg );
  else
     setType( wdl );

  setSelect(TRUE);                       // selected by default
  m_bNewDefinition = TRUE;               // this is a new Grammar
                                         // or Vocab definition
}


//------------------------------------------------------------------------------
//  ~GrammarDescriptor()
//------------------------------------------------------------------------------
//
//  Destructor for the GrammarDescriptor class.
//
//------------------------------------------------------------------------------
GrammarDescriptor::~GrammarDescriptor( void )
{
   m_vTypeString.Empty();
}


//------------------------------------------------------------------------------
//  acknowledgeNewDefinition()
//------------------------------------------------------------------------------
//
//  Indicates to the object that the New Definition state has been acknowledged.
//  resets the NewDefiniton flag.
//
//------------------------------------------------------------------------------
void GrammarDescriptor::acknowledgeNewDefinition()
{
  
   m_bNewDefinition = FALSE;
}


//------------------------------------------------------------------------------
//  acknowledgeStateChange()
//------------------------------------------------------------------------------
//
//  Indicates to the object that the state change has been acknowledged.
//  resets the stateChange flag.
//
//------------------------------------------------------------------------------
void GrammarDescriptor::acknowledgeStateChange()
{
  
   m_bStateChange = FALSE;
}


//------------------------------------------------------------------------------
//  setSelect()
//------------------------------------------------------------------------------
//
//  Set/Reset the selectFlag to indicate that the Grammar/Vocabulary is 
//  selected or deselected for speech recognition
//
//------------------------------------------------------------------------------
void GrammarDescriptor::setSelect(BOOL selectFlag)
{
  
   m_bSelected = selectFlag;
   m_bStateChange = TRUE;
}


//------------------------------------------------------------------------------
//  vocabularyName()
//------------------------------------------------------------------------------
//
//  Get the vocabulary name
//
//------------------------------------------------------------------------------
char const * GrammarDescriptor::vocabularyName()
{
  
   return m_szVocabularyName;
}


//------------------------------------------------------------------------------
//  fsgFileName()
//------------------------------------------------------------------------------
//
//  Get the grammar fsg file (or dynamic grammar/vocabulary) name
//
//------------------------------------------------------------------------------
char const * GrammarDescriptor::fsgFileName()
{

   return m_szFSGFileName;
}


//------------------------------------------------------------------------------
//  newDefinition()
//------------------------------------------------------------------------------
//
//  Is this a new grammar/vocabulary description
//
//------------------------------------------------------------------------------
BOOL GrammarDescriptor::newDefinition()
{
  
   return m_bNewDefinition;
}


//------------------------------------------------------------------------------
//  selected()
//------------------------------------------------------------------------------
//
// Is the grammar/vocabulary selected
//
//------------------------------------------------------------------------------
BOOL GrammarDescriptor::selected()
{
  
   return m_bSelected;
}


//------------------------------------------------------------------------------
//  stateChange()
//------------------------------------------------------------------------------
//
// Has the state of the grammar changed (ie selected/deselected)
//
//------------------------------------------------------------------------------
BOOL GrammarDescriptor::stateChange()
{
 
   return m_bStateChange;
}


//------------------------------------------------------------------------------
//  type()
//------------------------------------------------------------------------------
//
//  return the grammar/vocabulary type
//
//------------------------------------------------------------------------------
vocabularyTypes GrammarDescriptor::type()
{
  
   return m_vocabularyType;
}


//------------------------------------------------------------------------------
//  setType()
//------------------------------------------------------------------------------
//
//  Set the grammar/vocabulary type description
//
//------------------------------------------------------------------------------
void GrammarDescriptor::setType(vocabularyTypes aType)
{
  
   m_vocabularyType = aType;
  switch (aType)
  {
  
  case fsg:
     m_vTypeString = "fsg file";
     break;

  case wdl:
     m_vTypeString = "wdl file";
     break;

  case dynamic:
     m_vTypeString = "dynamic";
     break;

  case external:
     m_vTypeString = "external";
     break;
  }

}


//------------------------------------------------------------------------------
//  typeString()
//------------------------------------------------------------------------------
//
//  return a descriptive string associated with the grammar/vocabulary
//  type.
//
//------------------------------------------------------------------------------
const char *GrammarDescriptor::typeString()
{
  return m_vTypeString;
}

