// kioskDlg.cpp : implementation file
//

#include "stdafx.h"
#include "kiosk.h"
#include "kioskDlg.h"
#include <winreg.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKioskDlg dialog

CKioskDlg::CKioskDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CKioskDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CKioskDlg)
	m_answer = _T("");
	m_question = _T("");
	//}}AFX_DATA_INIT

	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

        // load our dynamic SmApi

        SmApiLoadLibrary();
}

void CKioskDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CKioskDlg)
	DDX_Text(pDX, IDC_ANSWER, m_answer);
	DDX_Text(pDX, IDC_QUESTION, m_question);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CKioskDlg, CDialog)
	//{{AFX_MSG_MAP(CKioskDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKioskDlg message handlers

BOOL CKioskDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
        // we need to initialize the engine 2 times,
        // first find out which default language we have,
        // then copy our word pool file and initalize again

	StartRecoEngine("");
        StopRecoEngine();
        StartRecoEngine(m_engine_languagecode);

        // now we present the user interface in the matching language
        PopulateDialog();

        // and now we enable the grammar and wait for voice input
        EnableGrammar();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CKioskDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CKioskDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


// here we get all speech recognition engine messages

BOOL CKioskDlg::OnCommand(WPARAM wParam, LPARAM lParam) 
{
   switch (LOWORD(wParam))
   {
      case ENGINE_CONNECTION_ID:
           return ProcessEngineMessage(lParam);
      default:
           return CDialog::OnCommand(wParam, lParam);
   }
}


// here, we take the dialog template from the .exe file,
// which does not have any hardcoded strings and
// then we load the appropriate language specific resource dll,
// get all the strings we need and put them in the dialog
// For more complex dialogs, we need to handle resizing issues,
// but for this sample we just made sure that the dialog controls
// are big enough ...

void CKioskDlg::PopulateDialog()
{
   HINSTANCE resdllhandle;
   CString   resdllname;
   CString   s;

   resdllname  = PROJECT_PREFIX;
   resdllname += m_ui_languagecode;
   resdllname += RESOURCE_DLL_SUFFIX;

   resdllhandle = ::AfxLoadLibrary(resdllname);
   ::AfxSetResourceHandle(resdllhandle);

   ::AfxFormatString1(s,IDS_TITLE,"");
   SetWindowText(s);
   ::AfxFormatString1(s,IDS_WELCOMEPROMPT,"");
   SetDlgItemText(IDC_WELCOMEPROMPT,s);
   ::AfxFormatString1(s,IDS_INSTRUCTIONPROMPT,"");
   SetDlgItemText(IDC_INSTRUCTIONPROMPT,s);
   ::AfxFormatString1(s,IDS_QUESTIONPROMPT,"");
   SetDlgItemText(IDC_QUESTIONPROMPT,s);
   ::AfxFormatString1(s,IDS_ANSWERPROMPT,"");
   SetDlgItemText(IDC_ANSWERPROMPT,s);
   m_answer = _T("");
   m_question = _T("");

   ::AfxSetResourceHandle(::AfxGetInstanceHandle());
   ::AfxFreeLibrary(resdllhandle);

   UpdateData(FALSE);
}

// here, we start or restart the recognition engine
// first time we will be called without a languagecode
// on subsequent calls we will get the desired languagecode in

void CKioskDlg::StartRecoEngine(char *languagecode)
{
        long rc;
        char *szVersion;
        int nNumArgs = 0;
        SmArg smArgs[30];
        SM_MSG reply;
        unsigned long  ulNumOfLangs;
        char**         apchLangs;
        char *p;

        if (strlen(languagecode) != 0)
        {
           // here we copy our word pool files to the appropriate place
           // note that this has to be done before connecting to the
           // engine !!!

           CopyPoolFile(languagecode);
        }

        rc = pSmApiVersionCheck(SM_API_VERSION_STRING, &szVersion);
        rc = pSmOpen(0, NULL);

        pSmSetArg(smArgs[nNumArgs], SmNapplicationName, PROJECT_PREFIX);               
        nNumArgs++;
        pSmSetArg(smArgs[nNumArgs], SmNrecognize, TRUE);                                 
        nNumArgs++;
        pSmSetArg(smArgs[nNumArgs], SmNnavigator, FALSE);                                
        nNumArgs++;
        pSmSetArg(smArgs[nNumArgs], SmNwindowHandle, this->GetSafeHwnd());                    
        nNumArgs++;
        pSmSetArg(smArgs[nNumArgs], SmNconnectionId, ENGINE_CONNECTION_ID);  
        nNumArgs++;

        // if we are called the first time, we use the defaults 
        // of the voicetype engine as set in the control panel

        if (strlen(languagecode) == 0)
           pSmSetArg(smArgs[nNumArgs], SmNuserId, SM_USE_CURRENT );
        else
           pSmSetArg(smArgs[nNumArgs], SmNuserId, m_userid );
          
        nNumArgs++;

        if (strlen(languagecode) == 0)
           pSmSetArg(smArgs[nNumArgs], SmNenrollId, SM_USE_CURRENT );
        else
           pSmSetArg(smArgs[nNumArgs], SmNenrollId, m_enrollid );
           
        nNumArgs++;

        if (strlen(languagecode) == 0)
           pSmSetArg(smArgs[nNumArgs], SmNtask, SM_USE_CURRENT );
        else
           pSmSetArg(smArgs[nNumArgs], SmNtask, m_task );

        nNumArgs++;

        rc = pSmConnect(nNumArgs, smArgs, &reply);

        pSmGetUserId( reply, &p );
        strcpy(m_userid,p);

        pSmGetEnrollId( reply, &p );
        strcpy(m_enrollid, p);

        pSmGetTask( reply, &p );
        strcpy(m_task, p);

        pSmGetLanguages( reply, &ulNumOfLangs, &apchLangs);
        strcpy(m_engine_languagecode, apchLangs[0]);

        // we keep track of 2 different language codes,
        // the voice input language of the recognition engine
        // (5-char, e.g. EN_US for US english   A N D  
        // the user interface language (we choose 2 characters,
        // e.g. US for US English.
        // For this sample we always want to synch these 2,
        // so we set the UI language equal to the engine language here

        strcpy(m_ui_languagecode, (m_engine_languagecode+3));

        rc = pSmMicOn(&reply);

        return;
}

// This sample has some words which are not in the standard shipped
// vocabulary, so we need to prepare customized add-on pool files
// (containing the pronounciations of the needed words).
// We do this with the Dictionary Builder Tool.
// These files then need to go into
//  <installdir>\vocabs\langs\<language>\pools
// The location where this actually needs to go is stored in the
// Registry, so we first get the Registry string, and then copy
// our pool files. For real apps this probably should be done
// at installation time ...

void CKioskDlg::CopyPoolFile(char *languagecode)
{
        HKEY     hkey;
        unsigned char   pathname[_MAX_PATH];
        DWORD    pathnamelen = _MAX_PATH;
        char *   path;
        char     path2[_MAX_PATH];
        char *   p1;
        int      n1;

        RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                     "SOFTWARE\\IBM\\VoiceType\\Engine\\Directories",
                     0,
                     KEY_READ,
                     &hkey);
        RegQueryValueEx(hkey,"DataPath",0,NULL,pathname,&pathnamelen);
        RegCloseKey(hkey);

        path = (char *)(&(pathname[0]));

        p1 = strstr(path,"%L");
        if (p1 != NULL)
        {
            n1 = (p1 - path);
            strncpy(path2,path,n1);
            path2[n1]='\0';
            strcat(path2,languagecode);
            strcat(path2,p1+2);
        }

        strcpy(path,PROJECT_PREFIX);
        strcat(path,languagecode+3);
        strcat(path,WORDPOOL_SUFFIX);

        strcat(path2,"\\");
        strcat(path2,path);

        CopyFile(path,path2,FALSE);

        return;
}


void CKioskDlg::StopRecoEngine()
{
        SM_MSG reply;
        long  rc = 0;

        rc = pSmDisconnect(0, NULL, &reply);
        rc = pSmClose();

        return;
}

// here we switch the engine to another language
// note that we don't try to switch users, so the 
// default user needs to have multiple enrollments for
// each language we want to use.

void CKioskDlg::SwitchRecoEngine(char *languagecode)
{
   SM_MSG reply;
   int    rc;
   unsigned long n;
   char  **ids;

   rc = pSmQueryEnrollIds(m_userid,NULL,languagecode,&reply);
  
   rc = pSmGetEnrollIds(reply,&n,&ids);

   if ( n != 0 )  // only switch language if id exists
   { 
      strcpy(m_enrollid,ids[0]);

      rc = pSmQueryTasks(languagecode,&reply);
      rc = pSmGetTasks(reply,&n,&ids);
      strcpy(m_task,ids[0]);

      rc = pSmDisconnect(0, NULL, &reply);

      StopRecoEngine();
      StartRecoEngine(languagecode);
   }
}

// here we enable our grammar 

void CKioskDlg::EnableGrammar(void)
{
        SM_MSG reply;
        int    rc;
        char   grammar[_MAX_PATH];

        GetCurrentDirectory(_MAX_PATH,grammar);
        strcat(grammar,"\\");
        strcat(grammar,PROJECT_PREFIX);
        strcat(grammar,(m_engine_languagecode+3));
        strcat(grammar,GRAMMAR_SUFFIX);

        rc = pSmDefineGrammar(GRAMMAR_NAME, grammar, 0, &reply);
        rc = pSmEnableVocab(GRAMMAR_NAME, &reply);
        rc = pSmRecognizeNextWord(&reply);

        return;
}


void CKioskDlg::DisableGrammar()
{
        SM_MSG reply;
        int rc;

        rc = pSmDisableVocab(GRAMMAR_NAME,&reply);

        return;
}

// here we process all our engine messages

BOOL CKioskDlg::ProcessEngineMessage(LPARAM lParam)
{
        SM_MSG reply;
        int rc;
        int nMessageType;

        pSmReceiveMsg (lParam, &reply);
        pSmGetMsgType (reply, &nMessageType);
        pSmGetRc(reply, &rc);

        switch (nMessageType)
        {
                case SM_CONNECT_REPLY:
                        break;

                case SM_FOCUS_GRANTED:
                        break;

                case SM_FOCUS_LOST:
                        break;

                case SM_RECOGNIZED_PHRASE:
                        ProcessRecognizedPhrase(reply);
                        break;

                case SM_RECOGNIZED_WORD:
                        rc = pSmRecognizeNextWord(&reply);
                        break;

                case SM_RECOGNIZED_TEXT:
                        break;

                case SM_REQUEST_MIC_ON:
                        break;

                case SM_REQUEST_MIC_OFF:
                        break;

                case SM_UTTERANCE_COMPLETED:
                        break;

                case SM_REPORT_ENGINE_ERROR:
                        break;

                default:
                        break;
        }

        return TRUE;
}

// ok, the user has said something, so the engine sent
// us a message, we now have to extract the phrase said
// and any annotation string attached. 

void CKioskDlg::ProcessRecognizedPhrase(SM_MSG reply)
{
        SM_WORD* pWords;
        ULONG nNumWords, nFlags;
        SM_MSG replySync;
        char languagecode[8];
        CString szRecoText = _T("");
        ULONG nAnnotations;
        SM_ANNOTATION *annotations;
        CString szAnnotation = _T("");
        int rc;
        ULONG nCounter;
        CString location = _T("");

        rc = pSmGetFirmWords(reply, &nNumWords, &pWords);

        if (rc == SM_RC_OK)
        {
           rc = pSmGetPhraseState(reply, &nFlags);

           if (rc == SM_RC_OK)
           {
              if (!(nFlags & SM_PHRASE_REJECTED))
			  {

	              nCounter = nNumWords;

		          while (nCounter > 0)
			      {
					szRecoText += " ";
					szRecoText +=(LPCTSTR)pWords->spelling ;
					pWords++;
					nCounter--;
				}

				pSmGetAnnotations(reply,&nAnnotations,&annotations);

				while (nAnnotations > 0)
				{
					if (annotations->annodata.string != NULL)
	                    szAnnotation += annotations->annodata.string;
					nAnnotations--;
					annotations++;
				}

				// now we have the annotation(s) and recognized text
				// parse them, we have 3 command types, depending
				// on the first 4 characters in the annotation:
				// 'EXIT' --> terminate
				// 'TOWN' --> switch to another town and another language
				// 'LOCN' --> answer question about some location

				if (strncmp(szAnnotation,"EXIT",4) == 0)
				{
					EndDialog(0);
				}
				else if (strncmp(szAnnotation,"TOWN",4) == 0)
				{
					strcpy(languagecode,szAnnotation.Mid(5,5));
					DisableGrammar();
					SwitchRecoEngine(languagecode);
					PopulateDialog();
					EnableGrammar();
				}
				else if (strncmp(szAnnotation,"LOCN",4) == 0)
				{
					location = szAnnotation.Mid(5);
					AnswerQuestion(szRecoText,location);
				}
			  }
			}
		}

        // we always need to have an SmRecognizeNextWord
        // pending for the recognition engine to continue listening

        rc = pSmRecognizeNextWord(&replySync);

        return;
}

// Here we just do some primitive data lookup
// we have a text file with questions and answers 
// and look if we find what the user wants


void CKioskDlg::AnswerQuestion(CString szQuestion,CString szAnnotation)
{
   m_question = szQuestion;
   SearchAnswer(szAnnotation);
   UpdateData(FALSE);

   return;
}


void CKioskDlg::SearchAnswer(CString szSearch)
{
   CStdioFile f1;
   CString    s;
   char       datafile[_MAX_PATH];

   strcpy(datafile,PROJECT_PREFIX);
   strcat(datafile,m_engine_languagecode+3);
   strcat(datafile,DATAFILE_SUFFIX);

   f1.Open(datafile, CFile::modeRead | CFile::typeText);

   m_answer = _T("???");

   while ( (f1.ReadString(s)) == TRUE)
   {
      if (s == szSearch)
      {
         f1.ReadString(s);
         m_answer = s;
         break;
      }
   }

   f1.Close();

   return;
}




