//------------------------------------------------------------------------------
//
// COPYRIGHT:
// ----------
// Copyright (C) International Business Machines Corp., 1996.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM
// product and is provided to you solely for the purpose of
// assisting you in the development of your applications.  The
// code is provided "AS IS", without warranty of any kind.  IBM
// shall not be liable for any damages arising out of your use of
// the sample code, even if they have been advised of the
// possibility of such damages.
//
//------------------------------------------------------------------------------
// SmDpView.cpp : implementation of the CSmDplusView class
//

#include "stdafx.h"
#include "SmDplus.h"                   // defines the MFC application class

#include "SmDpDoc.h"                   // MFC document class is not important
#include "SmDpView.h"                  // defines this MFC view class
#include "macrolst.h"

#define  VT_APP_NAME         "DictationPlus"  // app+ViaVoice specific stuff
#define  VT_CONNECT_ID       711              // app+ViaVoice specific stuff
 
#define  MLE_CONTROL_ID      765              // just a magic number


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSmDplusView

IMPLEMENT_DYNCREATE(CSmDplusView, CView)

BEGIN_MESSAGE_MAP(CSmDplusView, CView)
	//{{AFX_MSG_MAP(CSmDplusView)
	ON_WM_CREATE()
	ON_COMMAND(ID_VOICETYPE_BEGINDICTATION, OnVoicetypeBegindictation)
	ON_COMMAND(ID_VOICETYPE_STOPDICTATION, OnVoicetypeStopdictation)
	ON_COMMAND(ID_VOICETYPE_CONNECT, OnVoicetypeConnect)
	ON_COMMAND(ID_VOICETYPE_DISCONNECT, OnVoicetypeDisconnect)
	ON_COMMAND(ID_VOICETYPE_CORRECTSELECTION, OnVoicetypeCorrectselection)
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_WM_SHOWWINDOW()
	ON_COMMAND(IDD_DLG_VTDATA, OnDlgVtdata)
	ON_UPDATE_COMMAND_UI(ID_VOICETYPE_CONNECT, OnUpdateVoicetypeConnect)
	ON_UPDATE_COMMAND_UI(ID_VOICETYPE_DISCONNECT, OnUpdateVoicetypeDisconnect)
	ON_UPDATE_COMMAND_UI(ID_VOICETYPE_BEGINDICTATION, OnUpdateVoicetypeBegindictation)
	ON_UPDATE_COMMAND_UI(ID_VOICETYPE_STOPDICTATION, OnUpdateVoicetypeStopdictation)
	ON_UPDATE_COMMAND_UI(ID_VOICETYPE_CORRECTSELECTION, OnUpdateVoicetypeCorrectselection)
	ON_UPDATE_COMMAND_UI(ID_APP_MACROLIST, OnUpdateAppMacrolist)
	ON_COMMAND(ID_APP_MACROLIST, OnAppMacrolist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSmDplusView construction/destruction

CSmDplusView::CSmDplusView()
{
}

CSmDplusView::~CSmDplusView()
{
   OnVoicetypeDisconnect();                // do some clean-up before exit.
}

BOOL CSmDplusView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSmDplusView drawing

void CSmDplusView::OnDraw(CDC* pDC)
{
	CSmDplusDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CSmDplusView diagnostics

#ifdef _DEBUG
void CSmDplusView::AssertValid() const
{
	CView::AssertValid();
}

void CSmDplusView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSmDplusDoc* CSmDplusView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSmDplusDoc)));
	return (CSmDplusDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSmDplusView message handlers

int CSmDplusView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	return 0;
}


//------------------------------------------------------------------------------
//
//  This method deals with messages from the speech engine.
//
//------------------------------------------------------------------------------

BOOL CSmDplusView::OnCommand(WPARAM wParam, LPARAM lParam) 
{
   int hold;                           // a holder for MLE Selected index
   int selFirst;                       // index of first selected char in MLE
   int selLast;                        // index of last selected char in MLE
   VTWord * pWord;                     // ptr to word from VTProxy wordlist
   BOOL status;                        // return value

   status = FALSE;                     // no problem so far

   switch( wParam )
   {
      case VT_CONNECT_ID:              // how the app sees engine messages

         mledit.GetSel( selFirst, selLast );     // selected (infirm) location
         voicetype.setAppIndex( selFirst );      // next in wordlist added here

         // now tell VTProxy about this message to the application and let the
         // proxy decide what to do about it.  If the engine is telling the
         // app that it has reco'ed words then the proxy will add them to the 
         // wordlist.  Messages are stored for display later.

         voicetype.engineMessage( lParam );   // VTProxy deals with the engine

         // now the proxy has done it's thing, the app needs to talk to the 
         // proxy to see if any action is required such as displaying words
         // or messages to the user.

         if( voicetype.isWordWaiting() )   // display any new words from engine
         {
            pWord = voicetype.getLastDisplayedWord(); // start at 1st new word

            if ( pWord == NULL )       // no new words, must be first batch
               pWord = voicetype.getFirstWordNU();

            else                       // added to end of list
               pWord = voicetype.getNextWordNU( pWord );

            while ( pWord )            // more words to be displayed
            {
               // check if it is a macro
               VTDictationMacro *pmacro = m_macrodatabase->find(pWord->spelling);
               if ( pmacro == NULL)
               {
                  // display a word and a space
                  mledit.ReplaceSel( pWord->spelling );   
                  mledit.ReplaceSel( " " );
               }
               else
               {
                  // get macrotext
                  mledit.ReplaceSel ( pmacro->text() );
                  if (!(pmacro->joinright()))
                  {
                     mledit.ReplaceSel( " " );
                  }
               }
               
               // point to next word in wordlist
               pWord = voicetype.getNextWordNU( pWord );
            }

            mledit.GetSel( selFirst, selLast );  // caret is at end of text
            hold = selFirst;                     // remember index of first

            mledit.ReplaceSel( voicetype.getInfirmBuff() );  // dump infirm

            mledit.GetSel( selFirst, selLast );  // now we are at the end again
            mledit.SetSel( hold, selLast );      // highlite the infirm words

            voicetype.allWordsProcessed();       // tell proxy we got all words
         }

         break;

      default:
         status = CView::OnCommand(wParam, lParam);
   }

   if ( voicetype.isMessageWaiting() )
      MessageBox( voicetype.getMessage());        // tell the user whatever
 
   return status;
}


//------------------------------------------------------------------------------
//
//  This method connects the app to the engine.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnVoicetypeConnect() 
{
   voicetype.connect( VT_APP_NAME, m_hWnd, VT_CONNECT_ID, TRUE );  // save audio

   if ( voicetype.isMessageWaiting() )
      MessageBox( voicetype.getMessage());

   m_macrodatabase = new VTDictationMacroDatabase();
   m_macrodatabase->open(voicetype.getDefaultUserID(),
                         voicetype.getDefaultTask(),
                         voicetype.getLanguageCode());

}


//------------------------------------------------------------------------------
//
//  This method disconnects the app from the engine.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnVoicetypeDisconnect() 
{
   if ( voicetype.isConnected() )
   {
      voicetype.disconnect();

      if ( voicetype.isMessageWaiting() )
         MessageBox( voicetype.getMessage());
      m_macrodatabase->close();
      delete m_macrodatabase;
      m_macrodatabase = NULL;
   }
}


//------------------------------------------------------------------------------
//
//  This method starts the dictation process.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnVoicetypeBegindictation() 
{
   m_macrodatabase->update();

   voicetype.beginDictation( TRUE );

   if ( voicetype.isMessageWaiting() )
      MessageBox( voicetype.getMessage());
}


//------------------------------------------------------------------------------
//
//  This method stops the dictation process.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnVoicetypeStopdictation() 
{
   voicetype.stopDictation();

   if ( voicetype.isMessageWaiting() )
      MessageBox( voicetype.getMessage());
}


//------------------------------------------------------------------------------
//
//  This method corrects a word which the user has dictated, then selected.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnVoicetypeCorrectselection() 
{
   int selFirst;
   int selLast;
   VTWord * pVTWord;

   mledit.GetSel( selFirst, selLast );

   if ( selFirst == selLast )          // anything selected?
   {
      MessageBox( "Select one dictated word, then use \"Correct Selection\"." );
   }
   else
   {
	   pVTWord = voicetype.correctSelection( selFirst, selLast );

      if ( pVTWord )
      {
         mledit.ReplaceSel( pVTWord->spelling );
         mledit.ReplaceSel( " " );
         mledit.SetSel( selFirst, selFirst + pVTWord->spelling_size );
      }
   }
}


//------------------------------------------------------------------------------
//
//  This method keeps the Multi-Line edit control sized with the app.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnSize(UINT nType, int cx, int cy) 
{
   CRect rect;

	CView::OnSize(nType, cx, cy);
	
   GetClientRect( &rect );

   if ( IsWindow( mledit.m_hWnd ) )    // make sure MLE exists
	   mledit.MoveWindow( &rect, TRUE );    // size the MLE
}


//------------------------------------------------------------------------------
//
//  This method keeps the MLE in focus as it is the only intersting thing.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);
	
   if ( IsWindow( mledit.m_hWnd ) )  // make sure mledit has been created
	   mledit.SetFocus();             // focus always goes to edit control
}


//------------------------------------------------------------------------------
//
//  This method creates the MLE and sets it up.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	DWORD dwStyle = ES_AUTOVSCROLL | ES_MULTILINE ;
   CRect myrect;

	CView::OnShowWindow(bShow, nStatus);
	
   if ( !IsWindow( mledit.m_hWnd ) )
   {
      GetClientRect( &myrect );        // edit control will fill client area

      mledit.Create( dwStyle, myrect, this, MLE_CONTROL_ID );  // normal create

      // This sample will not even try to synchronize user typed text with
      // text recognized from the engine, the easiest way is to make the 
      // edit control read-only

      mledit.SetReadOnly( TRUE );      

      mledit.ShowWindow( SW_SHOW );    // normal display 
      mledit.SetFocus();               // set WINDOW focus (not speech focus)

      mledit.ReplaceSel( "(Tip: click on connect, then begin dictation)" );
      mledit.SetSel( 0, -1, TRUE );

   }
}


//------------------------------------------------------------------------------
//
//  This method creates a dialog that shows user<-->engine data.
//
//------------------------------------------------------------------------------

void CSmDplusView::OnDlgVtdata() 
{
   if ( voicetype.isConnected() )
   {
	   vtdata.m_AppAPIVer = SM_API_VERSION_STRING;
	   vtdata.m_InstAPIVer = voicetype.getInstAPIVer();
	   vtdata.m_DefaultUserID = voicetype.getDefaultUserID();
	   vtdata.m_DefaultEnrollID = voicetype.getDefaultEnrollID();
	   vtdata.m_DefaultTask = voicetype.getDefaultTask();
   
      vtdata.DoModal();	
   }
   else 
      MessageBox( "Must be connected to ViaVoice");
}


//------------------------------------------------------------------------------
//
//  These command U/I handlers update menu items and toolbar buttons as needed
//
//------------------------------------------------------------------------------

void CSmDplusView::OnUpdateVoicetypeConnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( !voicetype.isConnected() );
}

void CSmDplusView::OnUpdateVoicetypeDisconnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( voicetype.isConnected() );
}

void CSmDplusView::OnUpdateVoicetypeBegindictation(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( voicetype.isConnected() && !voicetype.isRecognizing() );
}

void CSmDplusView::OnUpdateVoicetypeStopdictation(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( voicetype.isConnected() && voicetype.isRecognizing() );
}

void CSmDplusView::OnUpdateVoicetypeCorrectselection(CCmdUI* pCmdUI) 
{
   VTWord * pWord;                     // ptr to word from VTProxy wordlist
   int selFirst, selLast;

   pWord = voicetype.getFirstWordNU();     // has anything been dictated yet?
   mledit.GetSel( selFirst, selLast );     // is something selected? selected 

	pCmdUI->Enable( voicetype.isConnected() 
                   && !voicetype.isRecognizing() 
                   && (selFirst-selLast != 0)
                   && pWord );
}

void CSmDplusView::OnUpdateAppMacrolist(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( voicetype.isConnected() );	
}

void CSmDplusView::OnAppMacrolist() 
{
	// TODO: Add your command handler code here
	CMacroList macrolist(NULL,m_macrodatabase);
	macrolist.DoModal();
}
