/******************************************************************************/
/*                                                                            */
/* COPYRIGHT:                                                                 */
/* ----------                                                                 */
/* Copyright (C) International Business Machines Corp., 1996.                 */
/*                                                                            */
/* DISCLAIMER OF WARRANTIES:                                                  */
/* -------------------------                                                  */
/* The following [enclosed] code is sample code created by IBM                */
/* Corporation.  This sample code is not part of any standard IBM product     */
/* and is provided to you solely for the purpose of assisting you in the      */
/* development of your applications.  The code is provided "AS IS",           */
/* without warranty of any kind.  IBM shall not be liable for any damages     */
/* arising out of your use of the sample code, even if they have been         */
/* advised of the possibility of such damages.                                */
/*                                                                            */
/******************************************************************************/
// smdict.c : Dictation Sample Program using SMAPI calls
//

/***************************************************************/
/* Compiler Options                                            */
/***************************************************************/
#define STRICT          

/***************************************************************/
/* Application defines                                         */
/***************************************************************/
#define NO_RC_REPORT 99999

/***************************************************************/
/* Include files                                               */
/***************************************************************/
#include <windows.h>    // Windows Main Include file
#include <stdio.h>      // Standard I/O routines
#include <stdarg.h>     // Defines ANSI-style macros for
                        //   variable argument functions
#include <string.h>     // String manipulation functions
#include <smapi.h>      // Speech APIs
#include "smdict.h"     // Our Sample Include file

/***************************************************************/
/* Function Prototypes                                         */
/***************************************************************/
LONG APIENTRY WndProc ( HWND, UINT, WPARAM, LONG );
int  connect_to_engine ( HWND );
int  disconnect_from_engine ( HWND );
int  log_message ( char *, int );
int  play_tags ( );
int  request_focus ( );
int  begin_dictation ( );
int  stop_dictation ( );
int  enable_vocab ( char * );


/***************************************************************/
/* Callback Functions Prototypes                               */
/***************************************************************/
SmHandler ConnectCB     ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler DisconnectCB  ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler SetCB         ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler MicOnCB       ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler MicOffCB      ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler EnableVocabCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler GetNextWordCB ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler NextWordCB    ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler RecoTextCB    ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler UtteranceCB   ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler ReqFocusCB    ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler PlayWordsCB   ( SM_MSG reply, caddr_t client, caddr_t call_data );
SmHandler PlayStatusCB  ( SM_MSG reply, caddr_t client, caddr_t call_data );

/***************************************************************/
/* Variables                                                   */
/***************************************************************/

// Application name
char app_name [ ] = "SMDict";

// Handles for the Buttons on the Main Window
HWND hwndConnect;
HWND hwndDisconnect;
HWND hwndFocus;
HWND hwndPlayback;
HWND hwndClearTags;
HWND hwndBeginDict;
HWND hwndStopDict;
HWND hwndStatusBar;

// Position within the message list
static int list_pos = 0;

// Array of Tags/words that you are able to playback
static unsigned long  tag_list [ 1024 ];

// Number of tags/words that are in the tag_list array
static unsigned short num_tags = 0;

// Handle to the Message Window
HWND msg_list;

// Handle to the Dictation Window
HWND reco_text;

// Character Strings used as arguments needed for the SmOpen call
char   userid[STR_SIZE]   = SM_USE_CURRENT;		// userid
char   enrollid[STR_SIZE] = SM_USE_CURRENT;		// enrollment id
char   taskid[STR_SIZE]   = SM_USE_CURRENT;		// task id


// -----------------------------------------------------------------------
// Application Main Entry Point
// -----------------------------------------------------------------------
int WINAPI WinMain ( HINSTANCE inst, HINSTANCE prev_inst,
                     LPSTR cmd_line, int cmd_show )
{
  HWND      hwnd;
  MSG       msg;
  WNDCLASS  wnd_class;

  SetErrorMode ( SEM_NOALIGNMENTFAULTEXCEPT );

  // Register the Window Class and Procedure
  if ( ! prev_inst )
  {
    wnd_class.style           = CS_HREDRAW | CS_VREDRAW;
    wnd_class.lpfnWndProc     = WndProc;
    wnd_class.cbClsExtra      = 0;
    wnd_class.cbWndExtra      = 0;
    wnd_class.hInstance       = inst;
    wnd_class.hIcon           = LoadIcon ( inst, MAKEINTRESOURCE ( IDI_SMDICT ) );
    wnd_class.hCursor         = LoadCursor ( NULL, IDC_ARROW );
    wnd_class.hbrBackground   = GetStockObject ( WHITE_BRUSH );
    wnd_class.lpszMenuName    = app_name;
    wnd_class.lpszClassName   = app_name;

    RegisterClass ( &wnd_class );
  }

  // Create the Main Window
  hwnd = CreateWindow ( app_name, "Sm Dictation Sample",
                        WS_DLGFRAME,
                        CW_USEDEFAULT, CW_USEDEFAULT,
                        620, 452,
                        NULL, NULL, inst, NULL );

  // Display the window
  ShowWindow ( hwnd, cmd_show );

  UpdateWindow ( hwnd );

  // Message Processing Loop
  while ( GetMessage ( &msg, NULL, 0, 0 ) )
  {
    TranslateMessage ( &msg );
    DispatchMessage ( &msg );
  }

  // Disconnect from the Speech Engine
  disconnect_from_engine ( hwnd );

  return ( msg.wParam );
}

// -----------------------------------------------------------------------
// Application's Main Window Procedure
// -----------------------------------------------------------------------
long PASCAL WndProc ( HWND hwnd, UINT msg, WPARAM wParam, LONG lParam )
{
 
  static PROC lpfn;
  static HANDLE  hInstance;

  HMENU       menu;
  int         x, y;
  int         rc;
  SM_MSG      reply;
  int         type;
  HANDLE      dummy;
  
  switch ( msg )
  {
    case WM_CREATE:
         hInstance = ( ( LPCREATESTRUCT ) lParam ) ->hInstance;
         x = 110;
         y = 346;
		 
		   // Only enable the Connect button to start with
		    
		 hwndConnect = CreateWindow ( "button", "Connect",
                                     WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                                     x, y, BUT_WIDTH, BUT_HEIGHT,
                                     hwnd,(HMENU) (IDB_CONNECT), hInstance, NULL );
		   
		 hwndDisconnect = CreateWindow ( "button", "Disconnect",
                                       WS_CHILD | WS_VISIBLE | WS_DISABLED | BS_PUSHBUTTON,
                                       x = x + BUT_WIDTH + 3, y, BUT_WIDTH, BUT_HEIGHT,
                                       hwnd,(HMENU) (IDB_DISCONNECT), hInstance, NULL );
		
		 hwndBeginDict = CreateWindow ( "button", "Begin Dict",
                                       WS_CHILD | WS_VISIBLE | WS_DISABLED | BS_PUSHBUTTON,
                                       x = x + BUT_WIDTH + 3, y, BUT_WIDTH, BUT_HEIGHT,
                                       hwnd,(HMENU) (IDB_BEGIN), hInstance, NULL );
		   
		 hwndStopDict = CreateWindow ( "button", "Stop Dict",
                                       WS_CHILD | WS_VISIBLE | WS_DISABLED | BS_PUSHBUTTON,
                                       x = x + BUT_WIDTH + 3, y, BUT_WIDTH, BUT_HEIGHT,
                                       hwnd,(HMENU) (IDB_STOP), hInstance, NULL );
		
       hwndFocus = CreateWindow ( "button", "Focus",
                                       WS_CHILD | WS_VISIBLE | WS_DISABLED | BS_PUSHBUTTON,
                                       x = 110 + (BUT_WIDTH / 2), y = y + BUT_HEIGHT + 3, 
                                       BUT_WIDTH, BUT_HEIGHT,
                                       hwnd,(HMENU) (IDB_FOCUS), hInstance, NULL );

		 hwndPlayback = CreateWindow ( "button", "Playback",
                                       WS_CHILD | WS_VISIBLE | WS_DISABLED | BS_PUSHBUTTON,
                                       x = x + BUT_WIDTH + 3, y, BUT_WIDTH, BUT_HEIGHT,
                                       hwnd,(HMENU) (IDB_PLAYBACK), hInstance, NULL );
		   
		 hwndClearTags = CreateWindow ( "button", "Clear Tags",
                                       WS_CHILD | WS_VISIBLE | WS_DISABLED | BS_PUSHBUTTON,
                                       x = x + BUT_WIDTH + 3, y, BUT_WIDTH, BUT_HEIGHT,
                                       hwnd,(HMENU) (IDB_CLEAR), hInstance, NULL );
		   

       dummy = ( HANDLE ) GetWindowLong ( hwnd, GWL_HINSTANCE );

		 // Create the Dictation Window 
       reco_text = CreateWindow ( "edit", NULL,
                                  WS_CHILD | WS_VISIBLE |
                                  WS_VSCROLL | WS_BORDER | ES_LEFT |
                                  ES_MULTILINE | ES_NOHIDESEL |
                                  ES_AUTOVSCROLL,
                                  5, 5, 603, 150,
                                  hwnd, (HMENU)IDX_TEXT, dummy, ( LPVOID ) NULL );

		 // Create the Messages Window
       msg_list = CreateWindow ( "listbox", NULL,
                                  WS_CHILD | WS_VISIBLE | LBS_STANDARD | 
                                  WS_HSCROLL | WS_VSCROLL,
                                  10, 160, 595, 180,
                                  hwnd, (HMENU)IDX_LIST, dummy, ( LPVOID ) NULL );

         return ( 0 );
         break;

    case WM_SETFOCUS:
         SetFocus ( reco_text );
         return ( 0 );
         break;

    case WM_COMMAND:
         menu = GetMenu ( hwnd );
         switch ( wParam )
         {

           case IDM_EXIT:
                SendMessage ( hwnd, WM_CLOSE, 0, 0L );
                return ( 0 );
                break;

           case IDM_ABOUT:
                MessageBox ( hwnd, 
                  "smdict  Version 4.1\nCopyright  IBM Corp. 1996, 1997\nAll rights reserved.\n\nThis is a sample program from the\n"
                  "IBM ViaVoice(tm) Developer Tools\nVersion 4.1.\n\n"
                  "ViaVoice is a trademark of IBM Corp.\n",
                  app_name, MB_ICONINFORMATION | MB_OK );
                return ( 0 );
                break;


           case IDB_CONNECT:
                // Connect Button pushed
                // Establish connection to the speech engine

                // Send message to Message Window 
                SendMessage ( msg_list, LB_RESETCONTENT, 0, 0L );

                // Set Message list position to zero
                list_pos = 0;

                // Process the Connect request
                rc = connect_to_engine ( hwnd );
                return ( 0 );
                break;

           case IDB_DISCONNECT:
                // Disconnect Button pushed, process request
                // Disconnect from the speech engine
                rc = disconnect_from_engine ( hwnd );
                return ( 0 );
                break;

           case IDB_FOCUS:
                // Focus Button pushed, process request
                // If another speech aware application is active, 
                // interactions with the speech engine are blocked.
                // We must obtain the speech focus before we can do
                // anything 
                rc = request_focus ( );

                if ( rc <= 0 )
                {
                  // Log message to Message Window
                  log_message ( "request_focus", rc );
                }

                return ( 0 );
                break;

           case IDB_PLAYBACK:
                // Playback Button pushed, process request
                rc = play_tags ( );

                if ( rc <= 0 )
                {
                  // Log message to Message Window
                  log_message ( "playback", rc );
                }

                return ( 0 );
                break;

           case IDB_CLEAR:
                // Clear Tags Button pushed

                // Set number of tags/words to zero
                num_tags = 0;

                // Change status of buttons on main window
                EnableWindow(hwndPlayback,FALSE);

                // Log message to Message Window
                log_message ( "Cleared Tag buffer", NO_RC_REPORT );
                return ( 0 );
                break;

           case IDB_BEGIN:
                // Begin Dictation Button pushed, process request
                rc = begin_dictation ( );
				
                if ( rc != SM_RC_OK )
                { 
                  // Log message to Message Window
                  log_message ( "begin_dictation", rc );
                } 

                return ( 0 );
                break;

           case IDB_STOP:
                // Stop Dictaton Button pushed, process request
                rc = stop_dictation( );
                if ( rc <= 0 )
                {
                  // Log message to Message Window
                  log_message ( "stop_dictation", rc );
                }

                return ( 0 );
                break;
			
           case SM_MESSAGE_ID:
                // Received message from Speech Engine

                // Request to receive a message from the Engine
                SmReceiveMsg ( lParam, & reply );
                // Retrieve the type of message that was received
                SmGetMsgType ( reply, & type );
                // Retrieve Return Code
                SmGetRc ( reply, & rc );

                // Call SmDispatch to dispatch our callback routine
                SmDispatch ( lParam );

                break;
         }
         break;


    case WM_DESTROY:
         PostQuitMessage ( 0 );
         return ( 0 );
         break;

  }

  return ( DefWindowProc ( hwnd, msg, wParam, lParam ) );
}

// -----------------------------------------------------------------------
// Log Message to the Message Window
// -----------------------------------------------------------------------
int log_message ( char * msg, int rc )
{
  char  buffer [ 132 ];


  if ( rc == NO_RC_REPORT )
     sprintf ( buffer, "%s", msg );
  else
  {
     if ( rc == SM_RC_OK )
        sprintf ( buffer, "%s rc = SM_RC_OK", msg );
     else
        sprintf ( buffer, "%s rc = %d: %s", msg, 
                  rc, SmReturnRcDescription( rc ) );
  }

  SendMessage ( msg_list, LB_INSERTSTRING, ( WPARAM ) -1, 
                ( LPARAM ) ( LPCTSTR ) buffer );

  SendMessage ( msg_list, LB_SETCURSEL, ( WPARAM ) list_pos, ( LPARAM ) 0 );

  list_pos = list_pos + 1;

  return ( 0 );
}


// -----------------------------------------------------------------------
// Display a word on the Dictation Window
// -----------------------------------------------------------------------
int display_word ( char * word )
{
  char buff[132];

  sprintf ( buff, "%s ", word );

  // Send an EM_REPLACESEL message to replace the current selection
  // in an edit control with the specified text.
  SendMessage ( reco_text, EM_REPLACESEL, 0, ( LPARAM ) ( LPCSTR ) buff );

  return ( 0 );
}


// -----------------------------------------------------------------------
// Display an infirm word on Dictation Window
// -----------------------------------------------------------------------
int insert_infirm ( char * infirm_buff )
{
  int  lc;
  int  ll;
  int  pos;
  int  infirm_start;
  int  infirm_len;
  int  infirm_end;
  int t;  

  infirm_len = strlen ( infirm_buff );

  // Send an EM_GETLINECOUNT message to retrieve the number of lines
  // in a multiline edit control. 
  lc = SendMessage ( reco_text, EM_GETLINECOUNT, ( WPARAM ) 0, ( LPARAM ) 0 );

  lc = lc - 1;     // EM_LINEINDEX is 0-based.

  // Send an EM_LINEINDEX message to retrieve the character index
  // of a line in a multiline edit control. The character index is 
  // the number of characters from the beginning of the edit control
  // to the specified line. 
  pos = SendMessage ( reco_text, EM_LINEINDEX, ( WPARAM ) lc, ( LPARAM ) 0 );

  if ( pos == -1 )
  {
    // Log message to Message Window
	log_message ( "GetLineIndex came back with -1", NO_RC_REPORT );

    return ( 0 );
  }
  // Send an EM_LINELENGTH message to retrieve the length of a
  // line, in characters, in an edit control. 

  ll = SendMessage ( reco_text, EM_LINELENGTH, ( WPARAM ) pos, ( LPARAM ) 0 );

  infirm_start = pos + ll;

  infirm_end = infirm_start + infirm_len;

  // The following two lines position the cursor at the end 
  // of the line
  t = SendMessage (reco_text,WM_GETTEXTLENGTH,(WPARAM) 0,(LPARAM) 0);
  SendMessage ( reco_text, EM_SETSEL, ( WPARAM ) t, ( LPARAM ) t );

  // Now we put the infirm text into the Dictation Window
  SendMessage ( reco_text, EM_REPLACESEL, 0, ( LPARAM ) ( LPCTSTR ) infirm_buff );

  // Select the text we just put in here
  SendMessage ( reco_text, EM_SETSEL, ( WPARAM ) infirm_start, ( LPARAM ) infirm_end );

  return ( 0 );
}


// -----------------------------------------------------------------------
//  Process the Connect Request
// -----------------------------------------------------------------------
int connect_to_engine ( HWND hwnd )
{
   char * InstAPIVer;
   static int first = TRUE;
   int    rc;
   SmArg  smargs [ 30 ];
   int    smc;

   if ( first )
   // First time this program connects to speech engine,
   // so establish a SM API connection
   {
      rc = SmApiVersionCheck( SM_API_VERSION_STRING, &InstAPIVer );

      if ( rc == SM_RC_WRONG_SM_VERSION )
      {
         log_message( "This application requires a newer", NO_RC_REPORT );
         log_message( "version of the speech engine.  Current version:", NO_RC_REPORT );
         log_message( InstAPIVer, NO_RC_REPORT  );

         return( rc );
      }

      else
      {

        // Set up arguments for SmOpen call	
        smc = 0;
        SmSetArg ( smargs [ smc ], SmNapplicationName, "SMDict" );      smc++;
        SmSetArg ( smargs [ smc ], SmNwindowHandle,    hwnd );          smc++;
        SmSetArg ( smargs [ smc ], SmNconnectionId,    SM_MESSAGE_ID ); smc++;

        rc = SmOpen ( smc, smargs );

        // Add callbacks for messages we support
        if (rc == SM_RC_OK)
           SmAddCallback ( SmNconnectCallback,       ConnectCB,     NULL );

   	  if (rc == SM_RC_OK)
           SmAddCallback ( SmNdisconnectCallback,     DisconnectCB,  NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNsetCallback,         SetCB,         NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNmicOnCallback,       MicOnCB,       NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNmicOffCallback,      MicOffCB,      NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNenableVocabCallback, EnableVocabCB, NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNrecognizeNextWordCallback,   GetNextWordCB, NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNrecognizedWordCallback,      NextWordCB,    NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNrecognizedTextCallback,      RecoTextCB,    NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNutteranceCompletedCallback,  UtteranceCB,   NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNrequestFocusCallback,        ReqFocusCB,    NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNplayWordsCallback,           PlayWordsCB,   NULL );

        if (rc == SM_RC_OK)
           rc = SmAddCallback ( SmNplayWordsStatusCallback,     PlayStatusCB,  NULL );

         // Log message to Message Window
         if (rc == SM_RC_OK)
            log_message ( "SmOpen() and Callbacks added OK", NO_RC_REPORT );
         else
            log_message ( "ERROR SmOpen() or adding Callbacks", rc );

         first = FALSE;

         if ( rc != SM_RC_OK )
         {
            return ( rc );
         }
      }
   }

   // Set up arguments for Sm Connect call
   smc = 0;

   SmSetArg ( smargs [ smc ], SmNuserId,       userid   );  smc++;
   SmSetArg ( smargs [ smc ], SmNenrollId,     enrollid );  smc++;
   SmSetArg ( smargs [ smc ], SmNtask,         taskid   );  smc++;
   SmSetArg ( smargs [ smc ], SmNrecognize,    TRUE     );  smc++;
  
   // Connect to the Speech Engine
   rc = SmConnect ( smc, smargs, SmAsynchronous );

   // Log message to Message Window
   log_message ( "Connect_to_engine: SmConnect()", rc );

   // Enable the saving of recorded audio during recognition
   SmSet ( SM_SAVE_AUDIO, TRUE, SmAsynchronous );
   
   return ( 0 );
}


// -----------------------------------------------------------------------
//  Process the Disconnect Request
// -----------------------------------------------------------------------
int disconnect_from_engine ( HWND hwnd )
{
  int    rc;

  stop_dictation( );

  // Close the communication connection with the speech engine
  rc = SmDisconnect ( 0, NULL, SmAsynchronous );

  // Log message to Message Window
  log_message ( "Discon_from_engine:SmDisconnect", rc );

  return ( 0 );
}


// -----------------------------------------------------------------------
// Process the Request Speech Focus Request
// -----------------------------------------------------------------------
int request_focus ( )
{
  int   rc;

  // Request speech focus
  rc = SmRequestFocus ( SmAsynchronous );

  // Log message to Message Window
  log_message ( "RequestFocus: SmRequestFocus()", rc );

  return ( 0 );
}


// -----------------------------------------------------------------------
//  Process Begin Dictation Request
// -----------------------------------------------------------------------
int begin_dictation ( )
{
  int    rc;
  
  // Turn on the microphone and start the audio stream
  rc = SmMicOn ( SmAsynchronous );
  
  // Log message to Message Window
  log_message ( "begin_dictation: SmMicOn()", rc );

  return (rc);
}

// -----------------------------------------------------------------------
// Process Stop Dictation Request
// -----------------------------------------------------------------------
int stop_dictation ( )
{
  int    rc;

  // Turn off the microphone
  rc = SmMicOff ( SmAsynchronous );

  // Log message to Message Window
  log_message ( "stop_dictation: SmMicOff()", rc );

  return ( 0 );
}


// -----------------------------------------------------------------------
// Enable the vocabulary name passed in 
// -----------------------------------------------------------------------
int enable_vocab ( char * name )
{
  SM_MSG reply;
  int    rc;
  char   buffer[132];

  // Temporarily halt recognition, this is needed before you 
  // enable/change a vocabulary
  rc = SmHaltRecognizer ( & reply );

  log_message ( "EnableVocab: SmHaltRecognizer()", rc );

  // Enable a vocabulary to be used by the speech engine
  // to decode speech to text during a recognition session
  rc = SmEnableVocab ( name, SmAsynchronous );

  sprintf ( buffer, "EnableVocab: SmEnableVocab(%s)", name );

  // Log message to Message Window
  log_message ( buffer, rc );

  return ( 0 );
}


// -----------------------------------------------------------------------
// Process the Playback request
// -----------------------------------------------------------------------
int play_tags ( )
{
  int    rc;
  char   buffer [ 132 ];

  // Play back the collection of words to be played back to the
  // user as an error correction aid
  rc = SmPlayWords ( num_tags, tag_list, SM_PLAY_WORDS_CONTIGUOUS,
                     SmAsynchronous );

  sprintf ( buffer, "PlayTags: SmPlayWords(%d)", num_tags );

  // Log message to Message Window
  log_message ( buffer, rc );

  return ( 0 );
}


// -----------------------------------------------------------------------
// Connect CallBack 
// Callback from SmConnect
// -----------------------------------------------------------------------
SmHandler ConnectCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "ConnectCB: SmConnect()", rc );

  // Reset the number of tags/words counter
  num_tags = 0;

  if ( rc == SM_RC_OK )
  {
     // Enable the text vocabulary
     enable_vocab ( "text" );
  }

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Disconnect Callback
// Callback from SmDisconnect
// -----------------------------------------------------------------------
SmHandler DisconnectCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "DisconnectCB: SmDisconnect()", rc );

  if ( rc == SM_RC_OK )
  {
     // Change status of buttons on main window
     EnableWindow(hwndDisconnect,FALSE);
     EnableWindow(hwndConnect,TRUE); 
     EnableWindow(hwndFocus,FALSE);
     EnableWindow(hwndPlayback,FALSE);
     EnableWindow(hwndClearTags,FALSE);
     EnableWindow(hwndBeginDict,FALSE);
  }
  
  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Enable Vocabulary Callback
// Callback from SmEnableVocab
// -----------------------------------------------------------------------
SmHandler EnableVocabCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code 
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "EnableVocabCB: SmEnableVocab()", rc );

  if ( rc == SM_RC_OK )
  {
     // Change state of buttons on main window
     EnableWindow(hwndDisconnect,TRUE);
     EnableWindow(hwndConnect,FALSE);
     EnableWindow(hwndFocus,TRUE);
     EnableWindow(hwndBeginDict,TRUE); 
  }

  return ( rc );
}


// -----------------------------------------------------------------------
// SmSet Callback
// Callback from SmSet
// -----------------------------------------------------------------------
SmHandler SetCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "SetCB: SmSet()", rc );

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Begin Dictation/Microphone On Callback
// -----------------------------------------------------------------------
SmHandler MicOnCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "MicOnCB:", rc );

  if ( rc == SM_RC_OK || rc == SM_RC_MIC_ALREADY_ON )
  {
    //  Display "Dictating.." on Dictation Window
    insert_infirm ( "Dictating..." );

    // Enable the recognition of the next word
    rc = SmRecognizeNextWord ( SmAsynchronous );
    
    log_message ( "MicOnCB: SmRecognizeNextWord()", rc );

    if ( rc == SM_RC_OK )
    {
      // Change status of buttons on main window
      EnableWindow(hwndBeginDict,FALSE);
      EnableWindow(hwndStopDict,TRUE);
      EnableWindow(hwndClearTags,FALSE);
      EnableWindow(hwndPlayback,FALSE);
    }
  }

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Stop Dictation/Microphone off Callback
// -----------------------------------------------------------------------
SmHandler MicOffCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "MicOffCB:", rc );

  // SM_RC_OK means the engine now knows this app wants speech 
  // focus and will give it to this app as soon as it can.
  //
  // SM_RC_FOCUS_GRANTED means this app already had focus,
  // this happens for example, when SmConnect is called and
  // this is the only SMAPI client application connected.

  if ( rc == SM_RC_OK || rc == SM_RC_MIC_ALREADY_OFF )
  {
    // Change the status of the buttons on the main window
    EnableWindow(hwndStopDict,FALSE);
    EnableWindow(hwndBeginDict,TRUE);
    EnableWindow(hwndClearTags,TRUE);
    EnableWindow(hwndPlayback,TRUE);
  }

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Focus Callback
// -----------------------------------------------------------------------
SmHandler ReqFocusCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;
  
  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "ReqFocusCB:", rc );

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Utterance Callback
// -----------------------------------------------------------------------
SmHandler UtteranceCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "UtteranceCB:", rc );

  // Remove "Dictating..." message off dictation window
  SendMessage ( reco_text, EM_REPLACESEL, ( WPARAM ) 0, ( LPARAM ) ( LPCTSTR ) "" );
  
  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Playback Callback
// -----------------------------------------------------------------------
SmHandler PlayWordsCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "PlayWordsCB:", rc );

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Play Status Callback
// -----------------------------------------------------------------------
SmHandler PlayStatusCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  char buffer [ 132 ];
  int  rc;
  int  status;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Retrieve the status from the reply structure
  SmGetStatus ( reply, & status );

  sprintf ( buffer, "PlayStatusCB: status = %d,", status );

  // Log message to Message Window
  log_message ( buffer, rc );

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Get Next Word Callback
// -----------------------------------------------------------------------
SmHandler GetNextWordCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "GetNextWordCB:", rc );

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Next Word Callback
// -----------------------------------------------------------------------
SmHandler NextWordCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  int  rc;

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "NextWordCB:", rc );

  return ( SM_RC_OK );
}


// -----------------------------------------------------------------------
// Reco Text Callback
// -----------------------------------------------------------------------
SmHandler RecoTextCB ( SM_MSG reply, caddr_t client_data, caddr_t call_data )
{
  char      buffer [ 132 ];
  int       rc;
  SM_WORD * firm;
  int       num_firm;
  int       i;
  SM_WORD * infirm;
  int       num_infirm;
  char      infirm_buff [ 255 ];

  // Retrieve the return code
  SmGetRc ( reply, & rc );

  // Log message to Message Window
  log_message ( "RecoTextCB:", rc );

  if ( rc != SM_RC_OK )
  {
    return ( SM_RC_OK );
  }

  // Retrieve the firm words
  rc = SmGetFirmWords ( reply, & num_firm, & firm );

  sprintf ( buffer, "RecoTextCB: SmGetFirmWords(%d)", num_firm );

  // Log message to Message Window
  log_message ( buffer, rc );

  // If there are no firm words yet, then we need to delete any infirm ones 
  // that are displayed 
  if ( num_firm == 0 )
  {
    SendMessage ( reco_text, EM_REPLACESEL, ( WPARAM ) 0, ( LPARAM ) ( LPCTSTR ) "" );
  }

  for ( i = 0 ; i < num_firm ; i++ )
  {
    display_word ( firm->spelling );

    tag_list [ num_tags++ ] = firm->tag;

    firm++;
  }

 
  // Retrieve infirm words
  rc = SmGetInfirmWords ( reply, & num_infirm, & infirm );

  sprintf ( buffer, "RecoTextCB: SmGetInfirmWords(%d)", num_infirm );

  // Log message to Message Window
  log_message ( buffer, rc );


  if ( num_infirm == 0 )
  {
    return ( 0 );
  }


  infirm_buff [ 0 ] = ' ';
  infirm_buff [ 1 ] = '\0';

  for ( i = 0 ; i < num_infirm; i++ )
  {
    strcat ( infirm_buff, infirm->spelling );
    strcat ( infirm_buff, " " );

    infirm++;
  }

  insert_infirm ( infirm_buff );

  return ( SM_RC_OK );
}

